"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useRequestStoreVars = exports.useRequestStore = void 0;
var react_1 = require("react");
var prostore_1 = require("@proscom/prostore");
var useConnectStore_1 = require("./useConnectStore");
var ProstoreContext_1 = require("./ProstoreContext");
/**
 * Attached component to the provided store and requests data based on the variables and options
 *
 * @param storeOrName - RequestStore to subscribe to, or its name for it to be injected from the context
 * @param variables - Request variables. If they change, request will be fired again
 * @param options - Additional request options. They are not observed
 * @param checkRequest - Optional function which may be used to override default behavior of determining
 *  when to fire the request
 */
function useRequestStore(storeOrName, variables, options, checkRequest) {
    if (options === void 0) { options = null; }
    if (checkRequest === void 0) { checkRequest = prostore_1.checkRequestState; }
    // Inject store from context
    var store = ProstoreContext_1.useContextStore(storeOrName);
    // Local copy of store state
    var _a = react_1.useState(store.state), state = _a[0], setState = _a[1];
    // Subscribe to store changes
    useConnectStore_1.useConnectStore(store, setState);
    // Check if we need to request data
    var queryCheck = checkRequest(state, variables);
    // Memoized callback to request data
    var funcsRef = react_1.useRef({});
    funcsRef.current.loadData = function () {
        return store.loadData(variables, options);
    };
    var loadData = react_1.useCallback(function () {
        return funcsRef.current.loadData();
    }, []);
    // Request the data if it is should be requested
    react_1.useEffect(function () {
        if (queryCheck.request) {
            // Error should not happen here, because
            // it is handled inside RequestStore,
            // but we attach the catch listener just in case
            loadData().catch(function (e) { return console.error(e); });
        }
    });
    // Return stuff which may be useful in the component
    return { check: queryCheck, state: state, load: loadData };
}
exports.useRequestStore = useRequestStore;
/**
 * The same as useRequestStore, but it uses variableCreator which is supplied to useMemo and
 * can be used to memoize query variables. This may be used when the default behavior of deeply
 * comparing variables is not preferable (i.e., it is not fast enough, or variables are not simple objects)
 *
 * @returns {{load, check, state}} - Current request state
 */
function useRequestStoreVars(storeOrName, variableCreator, args) {
    var variables = react_1.useMemo(variableCreator, args);
    return useRequestStore(storeOrName, variables);
}
exports.useRequestStoreVars = useRequestStoreVars;
//# sourceMappingURL=useRequestStore.js.map