import {
  DependencyList,
  useCallback,
  useEffect,
  useMemo,
  useRef,
  useState
} from 'react';
import {
  CheckRequestResult,
  checkRequestState,
  CheckRequestStateFn,
  IRequestState,
  RequestStore
} from '@proscom/prostore';
import { useConnectStore } from './useConnectStore';
import { useContextStore } from './ProstoreContext';

export interface UseRequestStoreResult<State> {
  check: CheckRequestResult;
  state: State;
  load: () => Promise<State>;
}

/**
 * Attached component to the provided store and requests data based on the variables and options
 *
 * @param storeOrName - RequestStore to subscribe to, or its name for it to be injected from the context
 * @param variables - Request variables. If they change, request will be fired again
 * @param options - Additional request options. They are not observed
 * @param checkRequest - Optional function which may be used to override default behavior of determining
 *  when to fire the request
 */
export function useRequestStore<Vars, Data, Options>(
  storeOrName: string | RequestStore<Vars, Data, Options>,
  variables: Vars,
  options: Partial<Options> = null,
  checkRequest: CheckRequestStateFn<Vars, Data> = checkRequestState
): UseRequestStoreResult<IRequestState<Vars, Data>> {
  // Inject store from context
  const store: RequestStore<Vars, Data, Options> = useContextStore(
    storeOrName
  ) as RequestStore<Vars, Data, Options>;

  // Local copy of store state
  const [state, setState] = useState(store.state);

  // Subscribe to store changes
  useConnectStore(store, setState);

  // Check if we need to request data
  const queryCheck = checkRequest(state, variables);

  // Memoized callback to request data
  const funcsRef = useRef<{
    loadData?: UseRequestStoreResult<IRequestState<Vars, Data>>['load'];
  }>({});
  funcsRef.current.loadData = () => {
    return store.loadData(variables, options);
  };
  const loadData = useCallback(() => {
    return funcsRef.current.loadData();
  }, []);

  // Request the data if it is should be requested
  useEffect(() => {
    if (queryCheck.request) {
      // Error should not happen here, because
      // it is handled inside RequestStore,
      // but we attach the catch listener just in case
      loadData().catch((e) => console.error(e));
    }
  });

  // Return stuff which may be useful in the component
  return { check: queryCheck, state, load: loadData };
}

/**
 * The same as useRequestStore, but it uses variableCreator which is supplied to useMemo and
 * can be used to memoize query variables. This may be used when the default behavior of deeply
 * comparing variables is not preferable (i.e., it is not fast enough, or variables are not simple objects)
 *
 * @returns {{load, check, state}} - Current request state
 */
export function useRequestStoreVars<Vars, Data>(
  storeOrName: string | RequestStore<Vars, Data, any>,
  variableCreator: () => Vars,
  args: DependencyList
) {
  const variables = useMemo(variableCreator, args);
  return useRequestStore(storeOrName, variables);
}
